'use strict';

const Boom = require('@hapi/boom');
const crypto = require('crypto');
const Joi = require('joi');
const createLink = require('./create-link');

const configSchema = Joi.object({
    key: Joi.string()
        .description('the secret used for link signaturs')
        .required(),
    failAction: Joi.func().description(
        'when set an autentication error will call the provided function failAction(request, h, reason)'
    ),
})
    .required()
    .description('the configuration options');

const plugin = (server, options) => {
    const settings = Joi.attempt(options, configSchema);

    const fail = async (request, h, reason) => {
        if (settings.failAction) {
            const customResult = await settings.failAction(request, h, reason);
            if (customResult) return customResult;
        }

        const err = Boom.unauthorized(reason, 'signedlink');
        err.output.payload.errorCode = reason;
        return h.unauthenticated(err);
    };

    // eslint-disable-next-line
    server.plugins.hapiAuthSignedlink = {
        createLink: url => createLink(url, settings.key),
    };
    const scheme = {
        authenticate: async (request, h) => {
            if (!request.query.auth) {
                return fail(request, h, 'E_AUTH_PARAM_MISSING');
            }

            const path = request.raw.req.url;
            const withoutAuth = path.replace(/[&?]auth=[0-9a-f]{64}$/, '');
            const hash = crypto.createHash('sha256');
            const sig = hash.update(`${withoutAuth}${settings.key}`).digest('hex');

            if (request.url.searchParams.get('auth') !== sig) {
                return fail(request, h, 'E_INVALID_SIGNATURE');
            }
            if (request.url.searchParams.get('expires_at')) {
                const d = new Date();
                const number = parseInt(request.query.expires_at, 10);
                if (d > number) {
                    return fail(request, h, 'E_LINK_EXPIRED');
                }
            }
            return h.authenticated({ credentials: { signedLink: true }, artifacts: { sig } });
        },

        payload: async (request, h) => h.continue,
        response: async (request, h) => h.continue,
    };

    return scheme;
};

exports.plugin = {
    pkg: require('../package.json'), //eslint-disable-line
    register: async server => {
        server.auth.scheme('signedlink', plugin);
    },
};
