'use strict';

const Hapi = require('@hapi/hapi');
const { expect } = require('chai');

const privateKey = 'test-key';
const bb = require('bluebird');

const hapiAuthSignedlink = require('../');
const createLink = require('../lib/create-link');

const tokenHandler = (request, h) => h.response('ok');

describe('hapi-auth-signedlink', () => {
    let server;

    const start = async (options = { key: privateKey }) => {
        server = new Hapi.Server({
            // debug: true,
        });
        await server.register(hapiAuthSignedlink);
        server.auth.strategy('default', 'signedlink', options);

        server.route([
            {
                method: 'POST',
                path: '/token',
                handler: tokenHandler,
                options: {
                    auth: 'default',
                },
            },
        ]);
    };

    it('returns HTTP 401 without auth', async () => {
        await start();
        const res = await server.inject({
            method: 'post',
            url: '/token',
        });
        expect(res.statusCode).to.equal(401);
    });

    it('returns HTTP 200 with valid auth param', async () => {
        await start();
        const url = createLink('/token', privateKey);
        console.log(url)

        const res = await server.inject({
            method: 'post',
            url,
        });
        expect(res.statusCode).to.equal(200);
    });

    it('handles urls with query params', async () => {
        await start();
        const url = createLink(`/token?email=${encodeURIComponent('test@test.com')}`, privateKey);
        const res = await server.inject({
            method: 'post',
            url,
        });
        expect(res.statusCode).to.equal(200);
    });

    it('fails with "expires_at" query param in past', async () => {
        await start();
        const d = new Date();

        await bb.delay(50);
        const url = createLink(`/token?expires_at=${d.getTime()}`, privateKey);
        const res = await server.inject({
            method: 'post',
            url,
        });
        expect(res.statusCode).to.equal(401);
    });

    it('success with "expires_at" query param in future', async () => {
        await start();
        const d = new Date();
        d.setFullYear(d.getFullYear() + 1);
        const url = createLink(`/token?expires_at=${d.getTime()}`, privateKey);
        const res = await server.inject({
            method: 'post',
            url,
        });
        expect(res.statusCode).to.equal(200);
    });

    it('registers "hapiAuthSignedlink" plugin', async () => {
        await start();
        expect(server.plugins.hapiAuthSignedlink.createLink).to.be.a('function');
    });

    it('"server.plugins.hapiAuthSignedlink.createLink(url)" produces valid signed link', async () => {
        await start();
        const url = server.plugins.hapiAuthSignedlink.createLink(`/token?email=${encodeURIComponent('test@test.com')}`);
        const res = await server.inject({
            method: 'post',
            url,
        });
        expect(res.statusCode).to.equal(200);
    });

    describe('with custom failAction', () => {
        it('can override default behaviour', async () => {
            await start({
                key: privateKey,
                failAction: async (request, h, reason) => {
                    return h.redirect('http://www.gogle.com').takeover();
                },
            });

            const res = await server.inject({
                method: 'post',
                url: '/token',
            });
            expect(res.statusCode).to.equal(302);
            expect(res.headers.location).to.equal('http://www.gogle.com');
        });

        it('uses failAction when "auth" parameter is missing', async () => {
            let reas;
            await start({
                key: privateKey,
                failAction: async (request, h, reason) => {
                    reas = reason;
                    return h.continue;
                },
            });

            const res = await server.inject({
                method: 'post',
                url: '/token',
            });
            expect(res.statusCode).to.equal(200);
            expect(reas).to.eql('E_AUTH_PARAM_MISSING');
        });

        it('uses failAction when link is expired', async () => {
            let reas;
            await start({
                key: privateKey,
                failAction: async (request, h, reason) => {
                    reas = reason;
                    return h.continue;
                },
            });
            const d = new Date();

            await bb.delay(50);
            const url = createLink(`/token?expires_at=${d.getTime()}`, privateKey);
            const res = await server.inject({
                method: 'post',
                url,
            });
            expect(res.statusCode).to.equal(200);
            expect(reas).to.eql('E_LINK_EXPIRED');
        });

        it('uses failAction when signature is invalid', async () => {
            let reas;
            await start({
                key: privateKey,
                failAction: async (request, h, reason) => {
                    reas = reason;
                    return h.continue;
                },
            });

            const url = `${createLink('/token', privateKey)}asdf`;

            const res = await server.inject({
                method: 'post',
                url,
            });
            expect(res.statusCode).to.equal(200);
            expect(reas).to.eql('E_INVALID_SIGNATURE');
        });

        it('when "failAction" returns "null" fallback to default behaviour', async () => {
            await start({
                key: privateKey,
                failAction: async () => null,
            });

            const url = `${createLink('/token', privateKey)}asdf`;

            const res = await server.inject({
                method: 'post',
                url,
            });
            expect(res.statusCode).to.equal(401);
        });

        it('when "failAction" returns "undefined" fallback to default behaviour', async () => {
            await start({
                key: privateKey,
                failAction: async () => { },
            });

            const url = `${createLink('/token', privateKey)}asdf`;

            const res = await server.inject({
                method: 'post',
                url,
            });
            expect(res.statusCode).to.equal(401);
        });
    });
});
